%%
%% Matlab Test Code - Side-Channel Statistical Toolbox
%% Copyright  Franois-Xavier Standaert
%% e-mail: <fstandae@uclouvain.be>
%%
%% This program is a free software: you can redistribute it 
%% and/or modify it under the terms of the GNU General Public 
%% License as published by the Free Software Foundation.
%%
%% It is distributed without any warranty of correctness nor
%% fintess for any particular purpose. See the GNU General 
%% Public License for more details.
%% 
%% <http://www.gnu.org/licenses/>
%%
%% Description: computation of MI(Y;L) with L=HW(Y)+N, 
%% and N a Gaussian noise, for different values of the noise  
%% variance, via numerical integration and sampling
%% 
%% Version 1 - February 2015

clear
warning off all
%--------------------------------------------------------------------------
% Main parameters
%--------------------------------------------------------------------------
n=8; % bitsize of Y
sampling_parameter=10; % set 5 for weak sampling, 20 for smoother sampling
%--------------------------------------------------------------------------

% vector of Hamming weights 
for i=1:2^n
    W(i)=hamming(i-1,n);
end
% 30 noise standard deviations (log scale)
log_sigma=linspace(-2,2,30);
log_var=2*log_sigma;

h = waitbar(0,'Please wait...');
for i=1:30 % for 30 different noise levels
    waitbar(i/30);
    sigma=10^log_sigma(i);
    HYL_int(i)=0; % H[Y|L] computed via integration
    for Wy=0:n % for all HW(y)   
        F = @(l) (normpdf(l,Wy,sigma)*log2(normpdf(l,Wy,sigma)/sum(normpdf(l,W,sigma))));
        HYL_int(i)=HYL_int(i)-nchoosek(n,Wy)/(2^n)*quadv(F,Wy-5*sigma,Wy+5*sigma);
    end
    HYL_sampling(i)=0; % H[Y|L] computed via sampling
    Nsamples=sampling_parameter*ceil(sigma^2); 
    for Wy=0:n % for all HW(y)   
        L=normrnd(Wy,sigma,1,Nsamples); % generation of a random leakage vector
        normterm=zeros(1,Nsamples); % normalizing term such that Pr[y|l]=pdf[l|y]/normterm
        for Wc=0:n % For all Hamming weight candidates
            if nchoosek(n,Wc)==1
                normterm=normterm+normpdf(L,Wc,sigma);
            else
                normterm=normterm+sum(repmat(normpdf(L,Wc,sigma),nchoosek(n,Wc),1));
            end
        end
        HYL_sampling(i)=HYL_sampling(i)-nchoosek(n,Wy)/(2^n)*mean(log2(normpdf(L,Wy,sigma)./normterm));
    end
end
close(h)
plot(log_var,log10(n-HYL_int),'LineWidth',2.5);
hold on;
plot(log_var,log10(n-HYL_sampling),'r--','LineWidth',2.5);
xlabel('log_{10}(noise variance)','FontSize',14);
ylabel('log_{10}(mutual information)','FontSize',14);
temp=legend(' numerical integration',' sampling');
LEG = findobj(temp,'type','text');
set(LEG,'FontSize',14);
set(gca,'fontsize',14);
title('Hamming weight leakage function','FontSize',14);
