%%
%% Matlab Test Code - Side-Channel Statistical Toolbox
%% Copyright  Franois-Xavier Standaert
%% e-mail: <fstandae@uclouvain.be>
%%
%% This program is a free software: you can redistribute it 
%% and/or modify it under the terms of the GNU General Public 
%% License as published by the Free Software Foundation.
%%
%% It is distributed without any warranty of correctness nor
%% fintess for any particular purpose. See the GNU General 
%% Public License for more details.
%% 
%% <http://www.gnu.org/licenses/>
%%
%% Description: computation of SR bounds based on the estimated NAMI and
%% the metric-based upper/lower bound from Poussier et al. (CARDIS 2015) 
%% 
%% Note: need to run compute_AMI first!
%% 
%% Version 1 - January 2016

clear
warning off all
load('NAMI.mat');
load('SNR.mat');
load('n.mat');
load('b.mat');

%--------------------------------------------------------------------------
% Main parameters
%--------------------------------------------------------------------------
Ns=16; % number of S-boxes
d=4; % order of the (perfect) masking scheme
msg_resolution=200; % number of intermediate measurement complexitites in the plot
time_resolution=Ns*n+1; % number of intermediate time complexities in the plot
q=round(1/NAMI(1)^d); % maximum number of measurements
%--------------------------------------------------------------------------
% Upper bound technical parameters (see CARDIS 2015 paper)
%--------------------------------------------------------------------------
iw=100; % inverse of the logarithmic bin width 
Nbins=iw*n+1; % number of bins in the logarithmic indexing 

mvect=logspace(log10(1),log10(q),msg_resolution);  % SR computation for log spaced number of measurements
hh = waitbar(0,'Please wait...');

% single S-box success rate computation
for m=1:msg_resolution
    waitbar((0*msg_resolution+m)/(7*msg_resolution));
    for c=1:2^n
        SR1(m,c)=max(1-(1-NAMI(c)^d)^(c*mvect(m)),+c/(2^n));
    end
end

% single S-box success rate derivatives computation
for m=1:msg_resolution
    waitbar((1*msg_resolution+m)/(7*msg_resolution));
    DSR(m,1)=SR1(m,1);
    for c=1:2^n-1
        DSR(m,c+1)=SR1(m,c+1)-SR1(m,c);
    end
end

% single S-box logarithmic indexing computation
log_vect=round(logspace(log10(1),log10(2^n),Nbins));
for m=1:msg_resolution
    waitbar((2*msg_resolution+m)/(7*msg_resolution));
    LI(m,:)=zeros(1,Nbins);
    for c=1:2^n
        temp1=c>=log_vect;
        temp2=c<=log_vect;
        temp=bitand(temp1,temp2);
        if sum(temp~=0)
            index=min(find(temp==1));
        else
            index=max(find(temp1==1));
        end
        LI(m,index)=LI(m,index)+DSR(m,c);
    end
end

% convolution of logarithmic indexed vectors for multiple S-boxes
for m=1:msg_resolution
    waitbar((3*msg_resolution+m)/(7*msg_resolution));
    LI_curr=LI(m,:);
    for Sbox=2:Ns
        LI_curr=conv(LI_curr,LI(m,:));
    end
    LI_conv(m,:)=LI_curr;
end

% success rate bound
for m=1:msg_resolution
    waitbar((4*msg_resolution+m)/(7*msg_resolution));
    SR_bound(m,1)=sum(LI_conv(m,1:iw));
    for i=2:Ns*n
        SR_bound(m,i)=SR_bound(m,i-1)+sum(LI_conv(m,(i-1)*iw+1:i*iw));
    end
    SR_bound(m,Ns*n+1)=SR_bound(m,Ns*n)+LI_conv(Ns*n*iw+1);
end
close(hh);

% security graph
tvect=[0:Ns*n];
subplot(2,1,1);
h=surf(mvect,tvect,1-SR_bound','EdgeColor','none');
set(get(h,'Parent'),'XScale','log');
view(2);
colormap(gray);
axis([1 q 0 Ns*n]);
ylabel('log_{2}(time complexity)','FontSize',14);
set(gca,'fontsize',14);
if b==1
    str=sprintf('MI bound, SR bound: SNR=%g, d=%i',SNR,d);
    title(str,'FontSize',14)
else
    str=sprintf('MI approximation, SR bound: SNR=%g, d=%i',SNR,d);
    title(str,'FontSize',14)
end
    

%--------------------------------------------------------------------------
% Lower bound technical parameters (see CARDIS 2015 paper)
%--------------------------------------------------------------------------
Nbins=50; % number of bins in the logarithmic subsampling (n+1<Nbins<2^n) 

clear('tvect');
hh = waitbar(0,'Please wait...');

% single S-box logarithmic downsampling computation
indices=round(logspace(log10(1),log10(2^n),Nbins));
for m=1:msg_resolution
    waitbar((5*msg_resolution+m)/(7*msg_resolution));
    for i=1:Nbins
        LSR(m,i)=SR1(m,indices(i));
    end
end

% logarithmic compbination of multiple S-boxes
log_indices=log2(indices);
for m=1:msg_resolution
    waitbar((6*msg_resolution+m)/(7*msg_resolution));
    SRn_old=LSR(m,:);
    Xn_old=log_indices;
    SRn_new=[];
    Xn_new=[];
    for Sbox=2:Ns
        clear('SRn_new');
        clear('Xn_new');
        [Xn_new,SRn_new]=comb_log(Xn_old,SRn_old,log_indices,LSR(m,:));
        clear('SRn_old');
        clear('Xn_old');
        SRn_old=SRn_new;
        Xn_old=Xn_new;
    end
    SR_heuristic(m,:)=SRn_new;
    tvect(m,:)=Xn_new;
end
close(hh);

%security graph
subplot(2,1,2);
h=surf(mvect,min(tvect),1-SR_heuristic','EdgeColor','none');
set(get(h,'Parent'),'XScale','log');
view(2);
colormap(gray);
axis([1 q 0 Ns*n]);
xlabel('number of measurements','FontSize',14);
ylabel('log_{2}(time complexity)','FontSize',14);
set(gca,'fontsize',14);
if b==1
    str=sprintf('MI bound, SR heuristic: SNR=%g, d=%i',SNR,d);
    title(str,'FontSize',14)
else
    str=sprintf('MI approximation, SR heuristic: SNR=%g, d=%i',SNR,d);
    title(str,'FontSize',14)
end
